# alarmclock.py
#
# Copyright (C) 2006 - Jon Oberheide <jon@oberheide.org>
# Copyright (C) 2006 - Fabien Carrion <fabien.carrion@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import string, sys, user, pickle
import pygtk, gtk, gtk.glade
import rhythmdb, rb
from configdata import ConfigData
from alarmsleep import AlarmSleep
from dailyalarmclock import DailyAlarmClock

ui_str = """
<ui>
  <menubar name="MenuBar">
    <menu name="ToolsMenu" action="Tools">
      <placeholder name="ToolsOps_5">
        <menuitem name="AlarmClock" action="AlarmClock"/>
      </placeholder>
    </menu>
  </menubar>
</ui>
"""

class AlarmClockPlugin(rb.Plugin):
    def __init__(self):
        rb.Plugin.__init__(self)
        self.configdata = ConfigData()
        self.configfile = user.home + '/.gnome2/rhythmbox/rb-alarmclock.config'
        for path in sys.path:
            try:
                self.gladefile = path + "/alarmclock.glade"
            except:
                pass
            else:
                break
        self.glade = gtk.glade.XML(self.gladefile)
        self.dailyalarm = DailyAlarmClock(self.configdata, self.glade)
        self.alarmsleep = AlarmSleep(self.configdata, self.glade)

        dic = { "on_spin_default_hour_value_changed" : self.dailyalarm.spin_default_hour_value_changed,
                "on_spin_default_minute_value_changed" : self.dailyalarm.spin_default_minute_value_changed,
                "on_check_volume_toggled" : self.dailyalarm.getFading().check_volume_toggled,
                "on_check_sleep_toggled" : self.alarmsleep.check_sleep_toggled,
                "on_check_sleep_aftersong_toggled" : self.alarmsleep.check_sleep_aftersong_toggled,
                "hide_dialog" : self.hide_dialog,
                "save_config" : self.save_config }
        self.glade.signal_autoconnect(dic)
        self.dialog = self.glade.get_widget("dialog")

    def activate(self, shell):
        data = {}
        manager = shell.get_player().get_property('ui-manager')
        action = gtk.Action('AlarmClock', _('_Alarm Clock'),
                            _("Configure alarm clock"),
                            'gnome-mime-text-x-python');
        action.connect('activate', self.show_dialog, shell)
        
        data['action_group'] = gtk.ActionGroup('AlarmClockPluginActions')
        data['action_group'].add_action(action)
        manager.insert_action_group(data['action_group'], 0)
        data['ui_id'] = manager.add_ui_from_string(ui_str)
        manager.ensure_update()
        
        shell.set_data('AlarmClockPluginInfo', data)

        self.dailyalarm.setShell(shell)
        self.alarmsleep.setShell(shell)
        self.load_config()
        self.dailyalarm.activate()
        self.alarmsleep.activate()

    def deactivate(self, shell):
        data = shell.get_data('AlarmClockPluginInfo')

        manager = shell.get_player().get_property('ui-manager')
        manager.remove_ui(data['ui_id'])
        manager.remove_action_group(data['action_group'])
        manager.ensure_update()

        shell.set_data('AlarmClockPluginInfo', None)

        self.dailyalarm.deactivate()
        self.alarmsleep.deactivate(True)

    def show_dialog(self, action, shell):
        self.load_config()
        self.dialog.show_all()
        self.dialog.grab_focus()

    def hide_dialog(self, *args):
        self.dialog.hide()
        return True

    def load_config(self):
        try: 
            self.configdata = pickle.load(open(self.configfile))
        except:
            self.configdata = ConfigData()
        self.dailyalarm.setConfigdata(self.configdata)
        self.alarmsleep.setConfigdata(self.configdata)
        self.dailyalarm.load_config()
        self.alarmsleep.load_config()

    def save_config(self, widget, data=None):
        self.dailyalarm.save_config()
        self.alarmsleep.save_config()
        pickle.dump(self.configdata, open(self.configfile, "w"))
        self.hide_dialog()

